<?php
if (!defined('PLX_ROOT')) { exit; };

/*
 * Plugin pour HighlightJS
 * https://github.com/highlightjs/highlight.js
 *
 * Permet la coloration syntaxique d'un extrait de code.
 * L'extrait doit être placé entre la paire de couples de balises :
 *
 * <pre><code lang="hljs php"> et </code></pre>
 *
 * Replace php with an other language if needed.
 * Available languages in common usage :
 * */

class kzHighlightJS extends plxPlugin {

	const ROOT = '/highlight.js/build/';
	const URL = PLX_PLUGINS . __CLASS__ . self::ROOT;
	const STYLES_LIST = __DIR__ . '/styles.txt';
	const PARAM1 = 'theme';

	/* Replace each '<' by entity inside <pre><code> tags in the content of an article or a static page */
	/* escaping '<' and '?>' avoids running PHP inside <pre><code> blocks in static pages  */
	const _PATTERN = <<< '_PATTERN'
'#(<pre[^>]*>\s*<code[^>]*>)\s*(.*?)\s*(</code>\s*</pre>)#is'
_PATTERN;
	const _REPLACE = <<< '_REPLACE'
function($m) { return $m[1].strtr($m[2], array('<' => '&lt;', '?>' => '?&gt;')).$m[3]; }
_REPLACE;
	const _PARSE_ARTICLE_BEGIN = <<< '_PARSE_ARTICLE_BEGIN'
<?php
$art['content'] = preg_replace_callback(
_PARSE_ARTICLE_BEGIN;
	const _PARSE_ARTICLE_END = <<< '_PARSE_ARTICLE_END'
	$art['content']
);
?>
_PARSE_ARTICLE_END;
	const PARSE_ARTICLE_CODE =
		self::_PARSE_ARTICLE_BEGIN ."\n\t" .
		self::_PATTERN . ",\n\t" .
		self::_REPLACE . ",\n\t" .
		self::_PARSE_ARTICLE_END;

	/* same play with a static page */
	const _PARSE_STATIC_BEGIN = <<< '_PARSE_STATIC_BEGIN'
<?php
$art['content'] = preg_replace_callback(
_PARSE_STATIC_BEGIN;
	const _PARSE_STATIC_END = <<< '_PARSE_STATIC_END'
	$art['content']
);
?>
_PARSE_STATIC_END;

	const _PARSE_STATIC_PART1 = <<< '_PARSE_STATIC_PART1'
<?php
if($this->plxMotor->aStats[ $this->plxMotor->cible ]['readable'] != 1) {
	echo '<p>'.L_STATICCONTENT_INPROCESS.'</p>';
	return true;
} else {
	$pattern =
_PARSE_STATIC_PART1;
	const _PARSE_STATIC_PART2 = <<< '_PARSE_STATIC_PART2'
	$file =
		PLX_ROOT .
		$this->plxMotor->aConf['racine_statiques'] .
		$this->plxMotor->cible . '.' .
		$this->plxMotor->aStats[ $this->plxMotor->cible ]['url'] .
		'.php';
	$content = file_get_contents($file);
	if(preg_match($pattern, $content)) {
		$new_content = preg_replace_callback(
			$pattern
_PARSE_STATIC_PART2;
	const _PARSE_STATIC_PART3 = <<< '_PARSE_STATIC_PART3'
$content
		);
		ob_start();
		eval('?>'.$new_content.'<?php');
		$output = ob_get_clean();
		eval($this->plxMotor->plxPlugins->callHook('plxShowStaticContent'));
		echo $output;
		return true;
	}
	return false;
}
?>
_PARSE_STATIC_PART3;
	const PARSE_STATIC_CODE =
		self::_PARSE_STATIC_PART1 .
		self::_PATTERN . ";\n" .
		self::_PARSE_STATIC_PART2 . ",\n\t\t\t" .
		self::_REPLACE . ",\n\t\t\t" .
		self::_PARSE_STATIC_PART3;

	public $themes = false;

	public function __construct($default_lang) {
		parent::__construct($default_lang);
		parent::setConfigProfil(PROFIL_ADMIN);
		if(!defined('PLX_ADMIN')) {
			parent::addHook('ThemeEndHead', 'ThemeEndHead');
	        parent::addHook('ThemeEndBody', 'ThemeEndBody');
	        parent::addHook('plxMotorParseArticle', 'plxMotorParseArticle');
	        parent::addHook('plxShowStaticContentBegin', 'plxShowStaticContentBegin');
		} elseif(basename($_SERVER['REQUEST_URI']) == 'parametres_plugin.php?p=' . __CLASS__) {
			// Just use for parametres_plugin.php in admin mode
			$this->themes = array();

			if(file_exists(self::STYLES_LIST)) {
				foreach(file(self::STYLES_LIST, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES) as $k) {
					$this->themes[$k] = ucwords(str_replace('-', ' ', $k));
				}
			}

			if(empty($this->themes)) {
				foreach(glob(__DIR__ . self::ROOT.'/demo/styles/*.css') as $t) {
					$k = basename($t, '.css');
					$this->themes[$k] = ucwords(str_replace('-', ' ', $k));
				}
				if(is_writable(__DIR__)) {
					file_put_contents(self::STYLES_LIST, implode(PHP_EOL, array_keys($this->themes)));
				}
			}
			parent::addHook('AdminTopEndHead', 'AdminTopEndHead');
			parent::addHook('AdminFootEndBody', 'ThemeEndBody');
		}
	}

	public function OnActivate() {
		// Lists again all available themes
		if(file_exists(self::STYLES_LIST) and is_writable(__DIR__)) {
			unlink(self::STYLES_LIST);
		}
	}

	/*
	public function OnDeActivate() { }
	public function onUpdate() { }
	 * */

	public function AdminTopEndHead() {
?>
	<style type="text/css">
		#<?php echo __CLASS__; ?>-container {
			height: calc(100vh - 15rem);
			margin: 0;
			padding: 0;
			display: flex;
			flex-direction: column;
		}
		#<?php echo __CLASS__; ?>-container > div.languages {
			position: fixed;
			top: 50vh;
			left: 50vw;
			transform: translate(-50%, -50%);
			display: none;
			background-color: #fff;
			border: 2px solid #444;
			padding: 0.3rem 1rem;
			max-width: 34rem;
			text-align: center;
		}
		#<?php echo __CLASS__; ?>-container > div.languages.active { display: block; }
		#<?php echo __CLASS__; ?>-container > div.languages span { 	background-color: #ddd; padding: 0.1rem 1rem; line-height: 1.7; border-radius: 0.3rem; }
		#<?php echo __CLASS__; ?>-container > div.languages button { margin-top: 0.7rem; }
		#form_<?php echo __CLASS__; ?> > div.in-action-bar * { min-width: 12rem; }
		div.<?php echo __CLASS__; ?>:not(.in-action-bar) { display: grid; grid-template-columns: 12rem auto; }
		div.<?php echo __CLASS__; ?> * { margin: 0; padding: 0; }
		div.<?php echo __CLASS__; ?> label { padding: 0.4rem 0.7rem 0 0; text-align: end; }
		pre.<?php echo __CLASS__; ?> { flex-grow: 1; margin-bottom: 0; }
		pre.<?php echo __CLASS__; ?> > code { height: 100%; }
	</style>
<?php

		// using alternate attribute avoids for loading all stylesheets if not required
		$currentStyle = $this->getParam(self::PARAM1);
		if(empty($currentStyle)) { $currentStyle = 'default'; }
		foreach($this->themes as $k=>$v) {
			$href = self::URL."demo/styles/$k.css";
?>
	<link rel="<?php if($k != $currentStyle) echo 'alternate '; ?>stylesheet" type="text/css" href="<?php echo $href; ?>" title="<?php echo $k; ?>" disabled />
<?php
		}
	}

	public function ThemeEndHead() {
		// Uncomment for debugging
		// file_put_contents('data/test-article.html', self::PARSE_ART_CODE);
		// file_put_contents('data/test-static.php', self::PARSE_STATIC_CODE);
		$param = trim($this->getParam(self::PARAM1));
		if(empty($param)) { $param = 'default'; }
		$href = 'demo/styles/'.$param.'.css';
		if(file_exists(__DIR__ . self::ROOT . $href)) {
?>
	<link rel="stylesheet" type="text/css" href="<?php echo self::URL . $href; ?>" />
	<style type="text/css">
		.hljs { line-height: 1.5em; }
	</style>
<?php
		}
	}

	public function ThemeEndBody() {
		// https://highlightjs.org/usage/
		// Also in use for config.php for selecting a theme

		$t = glob(__DIR__ . self::ROOT . 'demo/jquery*.js');
		if(empty($t)) {
			$folder = self::ROOT . 'demo/';
			echo <<< EOT
<!--
jQuery not found in $folder
-->
EOT;
			return;
		}
		$src = self::URL . 'demo/'.basename($t[0]);
?>
	<script type="text/javascript" src="<?php echo self::URL; ?>highlight.pack.js"></script>
	<script type="text/javascript">
		(function() { /* ----- <?php echo __CLASS__; ?> ----- */
			'use strict';

			// test jQuery
			if(typeof jQuery !== 'function') {
				const myScript = document.createElement('SCRIPT');
				myScript.type = 'text/javascript';
				myScript.src = '<?php echo $src; ?>';
				myScript.onload = function() {
					console.log('[<?php echo __CLASS__; ?>] <?php echo basename($t[0]); ?> loaded');
				};
				document.head.appendChild(myScript);
			}

			hljs.initHighlightingOnLoad();
		})();
	</script>
<?php
	}

	public function plxMotorParseArticle() {
		// Hack against PluXml when publishing;
		echo self::PARSE_ARTICLE_CODE;
	}

	public function plxShowStaticContentBegin() {
		// escape '<' for content inside <pre><code> tags for a static page
		echo self::PARSE_STATIC_CODE;
	}

}
?>
